%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Matlab template for plotting the Wigner function of a state defined in the Fock basis
%    Sebastian Wuester, Shivakant Tiwari; PHY 535 IISER Bhopal, 2019
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Nstates = 20;
Nsamples = 128; %points for plotting of the Wigner function

nvec = linspace(0,Nstates-1,Nstates); % our Fock basis

lambdamax = 8;			
dlambda = 2*lambdamax/Nsamples;
lambdavec=linspace(-lambdamax,lambdamax-dlambda,Nsamples);  %define the numerical grid used to sample lambda in the characteristic function

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%define coherent state
alpha = 2 + 1.0*i;
plotrange = 5;

psicoh = zeros(Nstates,1);
for nn=1:Nstates,
  psicoh(nn) = alpha^(nn-1)/sqrt(factorial(nn-1));
end
psicoh = exp(-abs(alpha)^2/2.0)*psicoh;

rho = zeros(Nstates,Nstates);   % define empty density matrix
for nn=1:Nstates,
   for mm = 1:Nstates,
      rho(nn,mm) = conj(psicoh(nn))*psicoh(mm);
   end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
aop =  zeros(Nstates,Nstates);  % define finite state space representation of destruction operator

for nn=1:Nstates-1,
  aop(nn,nn+1) = sqrt(nn);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

aopdagger = aop';  				% conjugate and transpose

%%% Write characteristic function
for nn = 1:Nsamples,
   for mm = 1:Nsamples,
     lambda = lambdavec(nn) + i*lambdavec(mm);
     D_op = expm(lambda*aopdagger - conj(lambda)*aop); % define displacement operator for current lambda
     chiW(nn,mm) = trace(rho*D_op);
   end
end

%%% filter pieces our where matrix exponential didn't converge
[LambdaRe LambdaIm] = ndgrid(lambdavec,lambdavec);
ModLambda = sqrt(LambdaRe.*LambdaRe + LambdaIm.*LambdaIm);
LambdaCut  =4;
Filter = (ModLambda < LambdaCut);
chiW = chiW.*Filter;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fourier transform to get Wigner function, create corresponding alpha grid
Wigner = fftshift(fft2(fftshift(chiW)))/pi^2;	
dlambda = lambdavec(2)-lambdavec(1);
alpha_max = pi/dlambda;
dalpha = 2*alpha_max/Nsamples;
alphavec = linspace(-alpha_max,alpha_max-dalpha,Nsamples)/2.0;   %%% 1/2 comes from def of Wignerfct	  

%%% need FFT shift ???

subplot(1,3,1)
hdl = pcolor(lambdavec,lambdavec,real(chiW));  %axis!!!
set(hdl,'EdgeColor','none')
xlabel(['Re[lambda]'])
ylabel(['Im[lambda]'])
title(['Char-fct, real'])

subplot(1,3,2)
hdl = pcolor(lambdavec,lambdavec,imag(chiW));  %axis!!!
set(hdl,'EdgeColor','none')
xlabel(['Re[lambda]'])
ylabel(['Im[lambda]'])
title(['Char-fct, imag'])

subplot(1,3,3)
hdl = pcolor(alphavec,alphavec,real(Wigner));  %axis!!!
set(hdl,'EdgeColor','none')
xlabel(['Re[alpha]'])
ylabel(['Im[alpha]'])
xlim([-plotrange plotrange])
ylim([-plotrange plotrange])
title(['Wigner-fct, Re'])
  
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lw=2;
fs=20;

lineobj = findobj('type', 'Line');
set(lineobj,'LineWidth', lw);

patchobj = findobj('type', 'Patch');
set(patchobj,'LineWidth', lw);

textobj = findobj('type', 'axes');
set(textobj,'LineWidth', lw);
set(textobj, 'fontsize', fs);
set(textobj, 'FontName', 'Times');
